"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTsconfig = void 0;
const utils_1 = require("@umijs/utils");
const path_1 = __importDefault(require("path"));
// @ts-ignore
const ts_transform_paths_1 = __importDefault(require("../../../../compiled/@zerollup/ts-transform-paths"));
/**
 * get parsed tsconfig.json for specific path
 */
function getTsconfig(cwd) {
    // use require() rather than import(), to avoid jest runner to fail
    // ref: https://github.com/nodejs/node/issues/35889
    const ts = require('typescript');
    const tsconfigPath = ts.findConfigFile(cwd, ts.sys.fileExists);
    if (tsconfigPath) {
        const tsconfigFile = ts.readConfigFile(tsconfigPath, ts.sys.readFile);
        return ts.parseJsonConfigFileContent(tsconfigFile.config, ts.sys, path_1.default.dirname(tsconfigPath));
    }
}
exports.getTsconfig = getTsconfig;
/**
 * get declarations for specific files
 */
async function getDeclarations(inputFiles, opts) {
    const output = [];
    // use require() rather than import(), to avoid jest runner to fail
    // ref: https://github.com/nodejs/node/issues/35889
    const ts = require('typescript');
    const tsconfig = getTsconfig(opts.cwd);
    if (tsconfig) {
        // check tsconfig error
        // istanbul-ignore-if
        if (tsconfig.errors.length) {
            throw new Error(`Error parsing tsconfig.json content: ${utils_1.chalk.redBright(ts.flattenDiagnosticMessageText(tsconfig.errors[0].messageText, '\n'))}`);
        }
        // enable declarationMap by default in development mode
        if (process.env.NODE_ENV === 'development' &&
            tsconfig.options.declaration &&
            tsconfig.options.declarationMap !== false) {
            tsconfig.options.declarationMap = true;
        }
        // remove paths which out of cwd, to avoid transform to relative path by ts-paths-transformer
        Object.keys(tsconfig.options.paths || {}).forEach((item) => {
            const pathAbsTarget = path_1.default.resolve(tsconfig.options.pathsBasePath, tsconfig.options.paths[item][0]);
            if (!pathAbsTarget.startsWith(opts.cwd)) {
                delete tsconfig.options.paths[item];
                utils_1.logger.debug(`Remove ${item} from tsconfig.paths, because it's out of cwd.`);
            }
        });
        const tsHost = ts.createCompilerHost(tsconfig.options);
        tsHost.writeFile = (fileName, declaration, _a, _b, sourceFiles) => {
            output.push({
                file: path_1.default.basename(fileName),
                content: declaration,
                sourceFile: sourceFiles[0].fileName,
            });
        };
        const program = ts.createProgram(inputFiles, tsconfig.options, tsHost);
        // using ts-paths-transformer to transform tsconfig paths to relative path
        // reason: https://github.com/microsoft/TypeScript/issues/30952
        // ref: https://www.npmjs.com/package/@zerollup/ts-transform-paths
        const result = program.emit(undefined, undefined, undefined, true, {
            afterDeclarations: [(0, ts_transform_paths_1.default)(program).afterDeclarations],
        });
        // check compile error
        // istanbul-ignore-if
        if (result.diagnostics.length) {
            throw new Error(`Error compiling declarations: ${utils_1.chalk.redBright(ts.flattenDiagnosticMessageText(result.diagnostics[0].messageText, '\n'))}`);
        }
    }
    return output;
}
exports.default = getDeclarations;
